/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package frontend.sjava;

import compbase.*;

/**
 * ExStr: constant string expression
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice
 *  version 080616 added filling of dependsOn, removed getCompInitConstObjectVrbl
 *  version 080614 adopted move of arrays from Clss to JUnit
 *  version 070918 made child of newly added ExConstInitObj
 *  version 070909 optimized signature of Expr.resolve, adopted changed ExAccVrbl
 *  version 070727 replaced exSubResolve by resolve as there is nothing done anymore
 *  version 070628 adopted changes for "Expr extends TypeRef"
 *  version 070511 removed invalid location assignment
 *  version 070114 reduced access level where possible
 *  version 070111 adapted change in printPos and compErr
 *  version 070106 removed genOutputAddr
 *  version 061203 optimized calls to printPos and compErr
 *  version 061202 optimized static modes
 *  version 061129 static TypeRef object moved dynamically to Context
 *  version 060607 initial version
 */

public class ExStr extends ExConstInitObj {
  protected String value;
  protected ExStr nextStr;
	
	protected ExStr(String is, int fid, int il, int ic) {
		super(fid, il, ic);
		value=is;
	}
	
	public boolean resolve(Unit unitContext, Mthd mthdContext, Context ctx) {
    JUnit myUnit;
    
    //enter the string in stringlist of unit
    if (!(unitContext instanceof JUnit)) {
      printPos(ctx, "java String in not-java unit is not supported");
      return false;
    }
    target.owner=myUnit=(JUnit)unitContext;
    target.minSize=-1;
    if (mthdContext!=unitContext.initStat && mthdContext!=unitContext.initDyna) dependsOn=mthdContext;
    nextStr=myUnit.strings;
    myUnit.strings=this;
    //set type and dest of parent
    baseType=T_QID;
    qid=ctx.stringType.qid;
    target.type=this;
    if ((mthdContext.modifier&Mthd.M_STAT)==0) {
      ensureClassContext=true;
      if (ctx.dynaMem) importedClass=unitContext.getRefUnit(unitContext, true);
    }
    constObject=true;
    return true;
  }
}
