/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package frontend.sjava;

import compbase.*;

/**
 * StBlock: block of statements
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice
 *  version 080610 removed conversion to JMthd as it is no longer required
 *  version 070913 added support for spread local variable declaration
 *  version 070909 optimized signature of Expr.resolve, made StBlock breakable
 *  version 070908 optimized signature of Stmt.resolve
 *  version 070905 added support for this(.)-calls
 *  version 070903 adopted changes in AccVar.checkNameAgainst*, now also checking locals against parameter
 *  version 070812 added support for double
 *  version 070701 added hint for architecture when local variables pushed
 *  version 070628 adopted changes for "Expr extends TypeRef"
 *  version 070527 added possibility to save instance context for inline array constructors
 *  version 070127 moved complex initialization of variables to Vrbl.genInitCode
 *  version 070114 reduced access level where possible
 *  version 070111 adapted change in printPos and compErr
 *  version 061203 optimized calls to printPos and compErr
 *  version 061128 added comments
 *  version 061027 optimized check for special names
 *  version 060621 inserted missing type-test for initialized variables
 *  version 060607 initial version
 */

public class StBlock extends StBreakable {
	protected Stmt stmts;
	
	protected StBlock(StBreakable io, StringList ila, int fid, int il, int ic) {
		super(io, ila, fid, il, ic);
	}
	
  protected boolean resolve(Unit unitContext, Mthd mthdContext, Context ctx) {
    return resolve(false, unitContext, mthdContext, ctx); //call not from mthd
  }
  
	protected boolean resolve(boolean isOutestInConstr, Unit unitContext, Mthd mthdContext, Context ctx) {
	  Stmt stmt;
    StExpr stex;
    ExCall call;
    Vrbl oldVarState;
	  
    oldVarState=mthdContext.vars;
		//check statements
		stmt=stmts;
    if (isOutestInConstr) { //in constructors, first statement has to be call to super
      if (!(stmt instanceof StExpr) || !((stex=(StExpr)stmt).ex instanceof ExCall)
          || !((call=(ExCall)stex.ex).id.name.equals(SJava.KEY_SUPER)
              || call.id.name.equals(SJava.KEY_THIS))) {
        compErr(ctx, "constructor without valid first this- or super-call");
        return false;
      }
      if (!call.callResolveThisSuper(unitContext, mthdContext, unitContext, mthdContext, ctx)) {
        ctx.out.print(" in call to this/super for constructor");
        return false;
      }
      stmt=stmt.nextStmt; //super-statement is resolved
    }
		while (stmt!=null) {
		  if (!stmt.resolve(unitContext, mthdContext, ctx)) return false;
		  stmt=stmt.nextStmt;
		}
    //everything done
		mthdContext.vars=oldVarState;
		return true;
	}
  
  protected void genOutput(Context ctx) {
    genOutput(ctx, false);
  }
	
	protected void genOutput(Context ctx, boolean saveInstContext) {
	  Stmt stmt;
	  
    //save instance context if required
    if (saveInstContext) ctx.arch.genSaveInstContext();
	  //generate output for statements
    breakDest=ctx.arch.getUnlinkedInstruction();
	  stmt=stmts;
	  while (stmt!=null) {
	    stmt.genOutput(ctx);
	    stmt=stmt.nextStmt;
	  }
    ctx.arch.appendInstruction(breakDest);
    breakDest=null;
	}
}
